function [x,n] = aitken(fun,dfun,x,tolx,maxiter)

%function [x,n] = aitken(fun,dfun,x,tolx,maxiter)
%
%cerca uno zero della funzione 'f' col metodo di accelerazione
%di Aitken
%
%fun e' la stringa della funzione
%dfun e' la stringa della derivata di f
%x_0 e' il punto di partenza del metodo
%tolx e' il valore della tolleranza della variabile indipendente (>0)
%maxiter e' il numero massimo di iterazioni consentite
%
%I valori di ritorno:
%x valore dello zero della funzione
%n numero di iterazioni eseguite
%
%ESEMPIO: newton('sin(x)','cos(x)',1,eps,100)

if tolx <= 0
    error('tolx deve avere un valore positivo')
end

f=inline(fun);
df=inline(dfun);
n=0;

fx=f(x);
dfx=df(x);
toly=abs(tolx*dfx);

while n<maxiter && abs(fx)>toly
  x1=x-fx/dfx;
  fx1=f(x1);
  dfx1=df(x1);
  x2=x1-fx1/dfx1;
  x=(x1^2-x*x2)/(2*x1-x-x2);
  fx=f(x);
  dfx=df(x);
  toly=abs(tolx*dfx);
  n=n+1;
end