function [x,n,maxiter] = bisez(s,a,b,tolx)

%function [x,n,maxiter] = bisez(s,a,b,tolx)
%
%trova uno zero della funzione 's' nell'intervallo (a,b)
%s e' la stringa della funzione
%a e b sono gli estremi dell'intervallo
%tolx e' il valore della tolleranza della variabile indipendente (>0)
%
%I valori di ritorno:
%x valore dello zero della funzione
%n numero di iterazioni eseguite
%maxiter limite superiore di iterazioni
%
%ESEMPIO: bisez('sin(x)',-0.5,1,eps)

f=inline(s);
fa=f(a);
fb=f(b);

% se f(a) o f(b) sono uguali a 0 esce subito

if fa*fb >= 0
  if fa==0
    x=a;
    n=0;
    return
  end

  if fb==0
    x=b;
    n=0;
    return
  end
  error('errore nella definizione di intervallo:f(a) e f(b) concordi')
end

% calcolo il numero di iterazioni massime necessarie
% per raggiungere la tolleranza richiesta

maxiter=ceil(log2(abs(b-a)/tolx));

% Se f(a) < 0 inverto l'intervallo
% in questo modo f(a) risultera' sempre >=  0.
% Questo semplifica il criterio di arresto.

if fa < 0
  t = a;
  a = b;
  b = t;
end

if tolx <= 0
  error('la tolleranza deve avere un valore positivo')
end

for n=1:maxiter
  x = (a+b)/2;
  fx = f(x);
      
%  if abs(fx) <= tolx * abs((fa-fb)/(a-b))
  if abs(fx) <= tolx * abs(15*x^14)
    return
  else
    if fx >= 0
      a=x;
      fa=fx;
    else
      b=x;
      fb=fx;
    end
  end
end