function [x,n,maxiter] = bisezPlot(s,a,b,tolx)

%function [x,n] = bisezPlot(s,a,b,tolx)
%
%trova uno zero della funzione 's' nell'intervallo (a,b)
%s e' la stringa della funzione
%a e b sono gli estremi dell'intervallo
%tolx e' il valore della tolleranza della variabile indipendente (>0)
%
%I valori di ritorno:
%x valore dello zero della funzione
%n numero di iterazioni eseguite
%
%ESEMPIO: bisez('sin(x)',-0.5,1,eps)

f=inline(s);
diff=abs(a-b);

Xs=a:diff/2000:b;
Ys=feval(f,Xs);
clf
hold on
fplot('0',[a b],'k');

plot(Xs,Ys,'b');


fa=f(a);
fb=f(b);

% se f(a) o f(b) sono uguali a 0 esce subito

if fa*fb >= 0
  if fa==0
    x=a;
    n=0;
    return
  end

  if fb==0
    x=b;
    n=0;
    return
  end
  error('errore nella definizione di intervallo:f(a) e f(b) concordi')
end

% calcolo il numero di iterazioni massime necessarie
% per raggiungere la tolleranza richiesta

maxiter=ceil(log2(abs(b-a)/tolx));

% Se f(a) < 0 inverto l'intervallo
% in questo modo f(a) risultera' sempre >=  0.
% Questo semplifica il criterio di arresto.


plot([a,a],[0,fa],'g');
plot([b,b],[0,fb],'g');
text(a+diff/50,fa+sign(fa)*diff/50,'a');
text(b-diff/50,fb+sign(fb)*diff/50,'b');
grid;
box off

if fa < 0
  t = a;
  a = b;
  b = t;
end

if tolx <= 0
  error('la tolleranza deve avere un valore positivo')
end



for n=1:maxiter
  x = (a+b)/2;
  fx = f(x);
  if(n<7)
    if fx == 0 
      sa=1;
    else
      sa = sign(fx);
    end
    
      
    text(x,fx+sa*diff/25,num2str(n));
  end
  plot([x,x],[0,fx],'k')
  
  %if abs(fx) <= tolx * abs((fa-fb)/(a-b))
  if abs(fx) <= tolx * abs(15*(x)^14)
    plot(Xs,Ys,'b');
    plot(x,0,'sr')
    return
  else
    if fx >= 0
      a=x;
      fa=fx;
    else
      b=x;
      fb=fx;
    end
  end
end
plot(Xs,Ys,'b');
plot(x,0,'sr')
