function [x,n] = newtonM(fun,dfun,x,tolx,maxiter,M)

%function [x,n] = newtonM(fun,dfun,x,tolx,maxiter,M)
%
%cerca uno zero della funzione 'f' col metodo di Newton
%nel caso che si conosca la moteplicit della radice
%fun e' la stringa della funzione
%dfun e' la stringa della derivata di f
%x_0 e' il punto di partenza del metodo
%tolx e' il valore della tolleranza della variabile indipendente (>0)
%maxiter e' il numero massimo di iterazioni consentite
%
%I valori di ritorno:
%x valore dello zero della funzione
%n numero di iterazioni eseguite
%
%ESEMPIO: newtonM('x.^4','4*x.^3',1,eps,100,4)

if tolx <= 0
    error('tolx deve avere un valore positivo')
end

f=inline(fun);
df=inline(dfun);
n=0;

fx=f(x);
dfx=df(x);
toly=abs(tolx*dfx);

while n<maxiter && abs(fx)>toly
  x=x-M*fx/dfx;
  fx=f(x);
  dfx=df(x);
  toly=abs(tolx*dfx);
  n=n+1;
end