function [x,n] = newtonPlotM(fun,dfun,x,tolx,maxiter,M)

%function [x,n] = newtonPlot(fun,dfun,x,tolx,maxiter)
%
%cerca uno zero della funzione 'f' col metodo di Newton 
%fun e' la stringa della funzione
%dfun e' la stringa della derivata di f
%x_0 e' il punto di partenza del metodo
%tolx e' il valore della tolleranza della variabile indipendente (>0)
%maxiter e' il numero massimo di iterazioni consentite
%
%I valori di ritorno:
%x valore dello zero della funzione
%n numero di iterazioni eseguite
%
%ESEMPIO: newton('sin(x)','cos(x)',pi/2,eps,100)

f=inline(fun);
df=inline(dfun);
n=0;

%disegno
minx=x;
maxx=x;
miny=0;
maxy=0;

clf
hold on;
fx=f(x);
plot([x x],[0 fx], 'r');
dfx=df(x);
toly=abs(tolx*dfx);

while n<maxiter && abs(fx)>toly
  oldx=x;
  oldy=fx;
  
  x=x-M*fx/dfx;
  if x>maxx
    maxx=x;
  end
  if x <minx
    minx=x;
  end

  
  fx=f(x);
  dfx=df(x);
  toly=abs(tolx*dfx);
  n=n+1;
  plot([x x],[0 fx], 'r');  
  plot([x oldx],[0 oldy], ':r');
  
end

diffx=maxx-minx;
diffy=maxy-miny;

Xs=minx:diffx/2000:maxx;
Ys=feval(f,Xs);


plot([minx maxx],[0 0],'k');
plot(Xs,Ys,'b');
plot(x,0,'sr');