function [x,n] = secanti(fun,dfx,x,tolx,maxiter)

%function [x,n] = secanti(fun,dfx,x,tolx,maxiter)
%
%cerca uno zero della funzione 'f' col metodo di Newton 
%fun e' la stringa della funzione
%dfx e' il valore della derivata in x
%x e' il punto di partenza del metodo
%tolx e' il valore della tolleranza della variabile indipendente (>0)
%maxiter e' il numero massimo di iterazioni consentite
%
%I valori di ritorno:
%x valore dello zero della funzione
%n numero di iterazioni eseguite
%
%ESEMPIO: secanti('sin(x)',cos(1),1,eps*10^2,1000)

if tolx <= 0
    error('tolx deve avere un valore positivo')
end

f=inline(fun);
n=0;

fx=f(x);
toly=abs(tolx*dfx);
x1=x-fx/dfx;
fx1=f(x1);
while n<maxiter && abs(fx1)>toly
  dfx=(fx1-fx)/(x1-x);
  x=x1;
  fx=fx1;
  x1=x-fx/dfx;
  fx1=f(x1);
  toly=abs(tolx*dfx);
  n=n+1;
end