function [s] = splinenat(x,f,xx)
%x e' il vettore delle ascisse di interpolazione
%f e' il vettore dei valori della funzione nelle ascisse del vettore x
%xx e' il vettore dei punti in cui vogliamo calcolare il valore della spline

n=length(x);
nf=length(f);

%controllo se x e f hanno la stessa dimensione
if n ~= nf, error('i vettori x ed f non hanno la stessa dimensione'), end

%controllo se n e' maggiore di 1
if n < 2 , error('il vettore x contiene meno di due valori!'), end

%per non fare confusione con il testo..
n = n - 1;

%h e' il vettore delle distanze tra i pinti di interpolazione
%ha dimensione n-1
h = diff(x);

%controllo si i punti di interpolazione sono stati assegnati in modo ordinato
%altrimenti li ordino
if(any(h<0)),
[x,i]=sort(x);
f=f(i);
h=diff(x);
end;

%controllo che nel vettore x cono compaiano ascisse coincidenti
if(all(h)==0), error('nel vettore x compaiono ascisse coincidenti'), end;

dd = diff(f)./h; %vettore delle differenze divise

b = 6 * diff(dd); %vettore dei termini noti


% d vettore della diagonale di D (n-1) elementi
% g vettore della sottodiagonale di g (n-2) elementi

d=zeros(n-1,1);
g=zeros(n-2,1);
d(1)=2*(h(1)+h(2));
for i=1:n-2
    g(i)=h(i+1)/d(i);
    d(i+1)=2*(h(i+1)+h(i+2))-g(i)*h(i+1);
end
M=b;
for i= 2:n-1
    M(i)=M(i)-g(i-1)*M(i-1);
end

M=M./d;
%keyboard
for i= n-2:-1:1
    M(i)=M(i)-g(i)*M(i+1);
end

M = [0;M;0]; % M e' di n+ 1 elementi (spline naturale)

C = dd - h.*diff(M)/6;

D = f(1:n) - M(1:n).*h.^2/6;

% ora abbiamo tutti i coefficienti necessari per il calolo della spline
% supponiamo che ci arrivino i punti ordinati xx

m=length(xx);

j=1;
for i=1:m
    while (xx(i)< x(j) & (j>1)); j=j-1; end
    while (xx(i)> x(j+1) & (j<n)); j=j+1; end % siamo nell' intervallo I_j
    
    dx1=xx(i)-x(j);
    dx2=x(j+1)-xx(i);
    
    s(i)=(M(j+1)*dx1^3+M(j)*dx2^3)/(6*h(j))+C(j)*dx1+D(j);
end
return
