
#define malloc	mxMalloc
#define free	mxFree

#include "gkls.c"
#include "rnd_gen.c"
#include <math.h>
#include <string.h>
#include "mex.h"

/* Operation types*/

#define	INFO		0
#define	ND_func		1
#define	D_func		2
#define	D2_func		3
#define	D_deriv		4
#define	D2_deriv1	5
#define	D2_deriv2	6
#define	D_gradient	7
#define	D2_gradient	8
#define	D2_hessian	9


/* Input Arguments */

#define	NUM_MINIMA_IN		prhs[0]
#define	GLOBAL_VALUE_IN		prhs[1]
#define	GLOBAL_DIST_IN		prhs[2]
#define	GLOBAL_RADIUS_IN	prhs[3]
#define	ACTION_IN			prhs[4]
#define	TEST_NUMBER_IN		prhs[5]
#define	TARGET_X1_IN		prhs[6]
#define	TARGET_X2_IN		prhs[7]
#define	DERIV_VAR1_IN		prhs[8]
#define	DERIV_VAR2_IN		prhs[9]

/* Output Arguments */

#define	RESULT	plhs[0]

int isSingleReal(const mxArray *mxptr)
{
	return (mxGetM(mxptr)*mxGetN(mxptr)) == 1 && mxIsDouble(mxptr) && !mxIsComplex(mxptr);
}

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{ 
    int test_number, action, deriv_var1, deriv_var2, i, j, m, n;
    GKLS_dim=2;

/*	Check for proper number of arguments */
    
    if (nrhs > 10) { 
		mexErrMsgTxt("Too many input arguments."); 
    }
    if (nrhs < 6) {
    		mexErrMsgTxt("Too few input arguments.");
    }
    if (nlhs > 1) {
		mexErrMsgTxt("Too many output arguments."); 
    } 

/*	Action check */
    if(mxIsChar(ACTION_IN)){
    		int strLength=mxGetN(ACTION_IN) + 1;
		char *actionStr=(char*)mxMalloc(strLength*sizeof(char));
		mxGetString(ACTION_IN, actionStr, strLength);
		
		if(strcmp(actionStr,"info")==0)
		{
			action=INFO;
		} else if(strcmp(actionStr,"ND_func")==0)
		{
			action=ND_func;
		} else if(strcmp(actionStr,"D_func")==0)
		{
			action=D_func;
		} else if(strcmp(actionStr,"D2_func")==0)
		{
			action=D2_func;
		} else if(strcmp(actionStr,"D_deriv")==0)
		{
			action=D_deriv;
		} else if(strcmp(actionStr,"D2_deriv1")==0)
		{
			action=D2_deriv1;
		} else if(strcmp(actionStr,"D2_deriv2")==0)
		{
			action=D2_deriv2;
		} else if(strcmp(actionStr,"D_gradient")==0)
		{
			action=D_gradient;
		} else if(strcmp(actionStr,"D2_gradient")==0)
		{
			action=D2_gradient;
		} else if(strcmp(actionStr,"D2_hessian")==0)
		{
			action=D2_hessian;
		} else {
			mexErrMsgTxt("Wrong action string");
		}
    } else if (!isSingleReal(ACTION_IN) || (action=floor(*mxGetPr(ACTION_IN)))<0 || action>9 ){ 
		mexErrMsgTxt("Wrong action number"); 
    }
    
    if( ((action==D_deriv || action==D2_deriv1) && nrhs!=9) || (action==D2_deriv2 && nrhs!=10))
    {
    		mexErrMsgTxt("Wrong input arguments number"); 
    }

/*	Arguments content check */

    if (!isSingleReal(NUM_MINIMA_IN) || (GKLS_num_minima=floor(*mxGetPr(NUM_MINIMA_IN)))<2 ){ 
		mexErrMsgTxt("Wrong minima number: must be a scalar >=2"); 
    }
    if (!isSingleReal(GLOBAL_VALUE_IN)){ 
		mexErrMsgTxt("Wrong global minimizer value: must be a scalar"); 
    } else {
	    	GKLS_global_value=*mxGetPr(GLOBAL_VALUE_IN);
    	}
    if (!isSingleReal(GLOBAL_DIST_IN) || (GKLS_global_dist=*mxGetPr(GLOBAL_DIST_IN))<=0 ){ 
		mexErrMsgTxt("Wrong distance: must be a scalar > 0"); 
    }
    if (!isSingleReal(GLOBAL_RADIUS_IN) || (GKLS_global_radius=*mxGetPr(GLOBAL_RADIUS_IN))<=0 ){ 
		mexErrMsgTxt("Wrong radius: must be a scalar > 0"); 
    }
    if (!isSingleReal(TEST_NUMBER_IN) || (test_number=floor(*mxGetPr(TEST_NUMBER_IN)))<1 || test_number>100){ 
		mexErrMsgTxt("Wrong test number: must be an integer 0 < i <= 100"); 
    }
    if (nrhs > 7 && ( (n=mxGetN(TARGET_X1_IN)) != mxGetN(TARGET_X2_IN) || (m=mxGetM(TARGET_X1_IN)) != mxGetM(TARGET_X2_IN))){ 
   		mexErrMsgTxt("Wrong target points matrix dimension: must have the same dimension"); 
    }
    if (action>6 && (n*m)!=1){
    		mexErrMsgTxt("Wrong target points matrix dimension for specified action: must be a sigle point");
    }
    if ((action==D_deriv || action==D2_deriv1 || action==D2_deriv2) && ((deriv_var1=floor(*mxGetPr(DERIV_VAR1_IN)))>GKLS_dim || !isSingleReal(DERIV_VAR1_IN))){
    		mexErrMsgTxt("Wrong first partial derivative index");
    }
    if (action==D2_deriv2 && ((deriv_var2=floor(*mxGetPr(DERIV_VAR2_IN)))>GKLS_dim || !isSingleReal(DERIV_VAR2_IN))){
    		mexErrMsgTxt("Wrong second partial derivative index");
    }
    
  
/*	Function domain allocation */
	if (GKLS_domain_alloc() != GKLS_OK){
		mexErrMsgTxt("Error allocating domain");
	}
	
	if(GKLS_arg_generate(test_number) != GKLS_OK) {
		mexErrMsgTxt("Internal error generating the function: check input data to be consistent");
	}
	
	double *x1,*x2, *res_matr, *temp;
    
    if(action!=INFO)
    {
        temp=(double *)mxMalloc(GKLS_dim*sizeof(double));
        x1=mxGetPr(TARGET_X1_IN);
        x2=mxGetPr(TARGET_X2_IN);
	}
        
	if(action==INFO){
        RESULT=mxCreateDoubleMatrix(GKLS_num_minima, GKLS_dim+2, mxREAL);
        res_matr=mxGetPr(RESULT);
        for(i=0; i<GKLS_num_minima; i++)
        {
            res_matr[i]=GKLS_minima.f[i];
            res_matr[i+GKLS_num_minima]=GKLS_minima.rho[i];
            for(j=0; j<GKLS_dim; j++)
            {
                res_matr[i+GKLS_num_minima*(j+2)]=GKLS_minima.local_min[i][j];
            }
        }
	} else if(action < 7) /* The function returns a matrix m*n */
	{
		RESULT=mxCreateDoubleMatrix(m, n, mxREAL);
		res_matr=mxGetPr(RESULT);
		
		for(i=0; i<(m*n); i++)
		{
			temp[0]=x1[i];
			temp[1]=x2[i];
			switch (action)
			{
				case ND_func:
					res_matr[i]=GKLS_ND_func(temp);
					break;
				case D_func:
					res_matr[i]=GKLS_D_func(temp);
					break;
				case D2_func:
					res_matr[i]=GKLS_D2_func(temp);
					break;
				case D_deriv:
					res_matr[i]=GKLS_D_deriv(deriv_var1, temp);
					break;
				case D2_deriv1:
					res_matr[i]=GKLS_D2_deriv1(deriv_var1, temp);
					break;
				case D2_deriv2:
					res_matr[i]=GKLS_D2_deriv2(deriv_var1, deriv_var2, temp);
					break;
			}
		}
		
	} else if(action <9) /* The function returns a vector */
	{
		temp[0]=x1[0];
		temp[1]=x2[0];
		RESULT=mxCreateDoubleMatrix(GKLS_dim, 1, mxREAL);
		res_matr=mxGetPr(RESULT);
		
		switch (action)
		{
			case D_gradient:
				GKLS_D_gradient(temp,res_matr);
				break;
			case D2_gradient:
				GKLS_D2_gradient(temp,res_matr);
				break;
		}
	} else				/* The function returns a matrix dim*dim */
	{
		temp[0]=x1[0];
		temp[1]=x2[0];
		RESULT=mxCreateDoubleMatrix(GKLS_dim, GKLS_dim, mxREAL);
		res_matr=mxGetPr(RESULT);
		
		for(i=0; i<GKLS_dim; i++)
			for(j=0; j<GKLS_dim; j++)
			{
				res_matr[i+GKLS_dim*j]=GKLS_D2_deriv2(i+1,j+1,temp);
			}
	}
	
    return;
    
}
