function res = cubicfitTPP_tolAlpha(p,x0,d,g)
%function res = cubicfitTPP_tolAlpha(p,x0,d,g)
%
% Funzione per la minimizzazione line search con il metodo del fit
% cubico, la convergenza è garantita mediante l'utilizzo del TPP
%
% res e' un cell array della forma {a,b} dove a e' il punto
% trovato, e b=f(a)

maxiter=getMaxiter(p);
tol=getTol(p);

g0=g'*d;
if(g0==0)
    f0=getValue(p,x0);
    res={x0,f0};
    return;
end;

% initialization of the TPP
step=abs(atan(g0)/g0);
[a0,a1,a2,f0,f1,f2]=initTPP(p,x0,d,step);
x=x0+a2*d;
fx=f2;
g1=getGrad(p,x0+a1*d)'*d;
g2=getGrad(p,x0+a2*d)'*d;
gx=g2;
n=0;

while n<maxiter && a2-a0>tol
u1=g1+g2-3.0*((f1-f2)/(a1-a2));
    sq=u1^2-g1*g2;
    if (sq>0)
        u2=sqrt(sq);
    else
        u2=0;
    end
    a3=a2-(a2-a1)*((g2+u2-u1)/(g2-g1+2*u2));
    
    x=x0+a3*d;
    fx=getValue(p,x);
    gx=getGrad(p,x)'*d;
    [a0,a1,a2,f0,f1,f2,g0,g1,g2]=nextTPPGrad(a0,a1,a2,a3,f0,f1,f2,fx,g0,g1,g2,gx);
    n=n+1;
end
x=x0+a1*d;
res={x,f1};

return;


