function res = goldstein(p,x,d,gx)
%function res = goldstein(p,x,d,gx)
%
% Funzione per la minimizzazione line search con il criterio di Goldstein
%
% res e' un cell array della forma {a,b} dove a e' il punto
% trovato, e b=f(a)

epsilon=getEpsilonArmijo(p);
if (epsilon=='')
    epsilon=.5;
end
eta=getEtaArmijo(p);
if (eta=='')
    eta=2;
end

alpha=0.1;
maxiter=getMaxiter(p);
tol=getTol(p);

phi0=getValue(p,x);
epsilondphi0=d'*gx*epsilon;
antiepsilondphi0=d'*gx*(1-epsilon);
xalpha=x+alpha*d;
phiAlpha=getValue(p,xalpha);
minAlpha=0;
minAlphaPhi=phi0;
maxAlpha=alpha;
maxAlphaPhi=phiAlpha;
while phiAlpha <= phi0+epsilondphi0*alpha;
    minAlpha=alpha;
    minAlphaPhi=phiAlpha;
    alpha=eta*alpha;    
    xalpha=x+alpha*d;
    phiAlpha=getValue(p,xalpha);
    maxAlpha=alpha;
    maxAlphaPhi=phiAlpha;
end

while phiAlpha > phi0+epsilondphi0*alpha;
    maxAlpha=alpha;
    maxAlphaPhi=phiAlpha;
    alpha=alpha/eta;    
    xalpha=x+alpha*d;
    phiAlpha=getValue(p,xalpha);
    minAlpha=alpha;
    minAlphaPhi=phiAlpha;
end

% a questo punto abbiamo che la condizione è soddisfatta senz'altro in un punto fra minalpha e
% maxalpha, partiamo con una ricerca con il metodo di bisezione
if ((maxAlphaPhi <= phi0+epsilondphi0*maxAlpha) && (maxAlphaPhi>phi0+antiepsilondphi0*maxAlpha))
    res={maxAlpha,maxAlphaPhi};
    return;
end

if ((minAlphaPhi <= phi0+epsilondphi0*maxAlpha) && (minAlphaPhi>phi0+antiepsilondphi0*minAlpha))
    res={minAlpha,minAlphaPhi};
    return;
end

n=0;
while ((n<maxiter)&&(maxAlpha-minAlpha>tol))
    alpha=(maxAlpha+minAlpha)/2.0;
    xalpha=x+alpha*d;
    phiAlpha=getValue(p,xalpha);
    % if goldstein rule holds
    if (phiAlpha>phi0+antiepsilondphi0*alpha)
        if (phiAlpha <= phi0+epsilondphi0*alpha)
            % both rules hold: found        
            break;
        else
            maxAlpha=alpha;
            maxAlphaPhi=phiAlpha;
        end
    else
        minAlpha=alpha;
        minAlphaPhi=phiAlpha;
    end
    n=n+1;
end

xalpha=x+alpha*d;
res={xalpha,phiAlpha};
return;






