function res = newtonTPP_goldstein(p,x0,d,gx)
%function res = newtonTPP_goldstein(p,x0,d,gx)
%
% Funzione per la minimizzazione line search con il metodo di Newton con TPP
%
% res e' un cell array della forma {a,b} dove a e' il punto
% trovato, e b=f(a)

maxiter=getMaxiter(p);
epsilon=getEpsilonArmijo(p);
if (epsilon=='')
    epsilon=.5;
end
phi0=getValue(p,x0);
epsilondphi0=d'*gx*epsilon;
antiEpsilondphi0=d'*gx*(1-epsilon);

gd=gx'*d;
if(gd==0)
    f0=getValue(p,x0);
    res={x0,f0};
    return;
end;

n=0;

% initialization of the TPP
step=abs(atan(gd)/gd);
[alfa0,alfa1,alfa2,f0,f1,f2]=initTPP(p,x0,d,step);


g0=gx;
g1=getGrad(p,x0+alfa1*d);
g2=getGrad(p,x0+alfa2*d);

H0=getHess(p,x0+alfa0*d);
H1=getHess(p,x0+alfa1*d);
H2=getHess(p,x0+alfa2*d);
while n<maxiter && ((f1 > phi0+epsilondphi0*alfa1) || (f1 <= phi0+antiEpsilondphi0*alfa1 ))
      gd=g1'*d;
      gamma=d'*H1*d;
      if gamma==0
          break;
      end      
      delta=gd/gamma;
      alfak=alfa1-delta;
      xk=x0+alfak*d;
      fk=getValue(p,xk);      
      gk=getGrad(p,xk);
      Hk=getHess(p,xk);      
      [alfa0,alfa1,alfa2,f0,f1,f2,g0,g1,g2,H0,H1,H2]=nextTPPHess(alfa0,alfa1,alfa2,alfak,f0,f1,f2,fk,g0,g1,g2,gk,H0,H1,H2,Hk);
      n=n+1;
end

res={x0+alfa1*d,f1};
return;



