function res = quadfitTPP_goldstein(p,x0,d,g0)
%function res = quadfitTPP_goldstein(p,x0,d,g0)
%
% Funzione per la minimizzazione line search con il metodo del fit
% quadratico. La convergenza è garantita con l'uso del TPP
%
% res e' un cell array della forma {a,b} dove a e' il punto
% trovato, e b=f(a)

maxiter=getMaxiter(p);

epsilon=getEpsilonArmijo(p);
if (epsilon=='')
    epsilon=.5;
end
phi0=getValue(p,x0);
epsilondphi0=d'*g0*epsilon;
antiEpsilondphi0=d'*g0*(1-epsilon);

g=g0'*d;
if(g==0)
    f0=getValue(p,x0);
    res={x0,f0};
    return;
end;

% initialization of the TPP
step=abs(atan(g)/g);
[a0,a1,a2,f0,f1,f2]=initTPP(p,x0,d,step);


n=0;
while n<maxiter && ((f1 > phi0+epsilondphi0*a1) || (f1 <= phi0+antiEpsilondphi0*a1 ))
    f0f1=(f1-f0)/(a1-a0);
    f1f2=(f2-f1)/(a2-a1);
    f0f1f2=(f1f2-f0f1)/(a2-a0);
    a3=(a0+a1)/2-f0f1/(2*f0f1f2);    
    f3=getValue(p,x0+a3*d);
    [a0,a1,a2,f0,f1,f2]=nextTPP(a0,a1,a2,a3,f0,f1,f2,f3);
    n=n+1;
end
x=x0+a1*d;
fx=getValue(p,x);
res={x,fx};

return;

