function res = secantTPP_armijo(p,x,d,gx0)
%function res = secantTPP_armijo(p,x,d,gx0)
%
% Funzione per la minimizzazione line search con il metodo delle secanti.
% La convergenza è garantita dall'uso del TPP
%
% res e' un cell array della forma {a,b} dove a e' il punto
% trovato, e b=f(a)


maxiter=getMaxiter(p);
epsilon=getEpsilonArmijo(p);
if (epsilon=='')
    epsilon=.5;
end
eta=getEtaArmijo(p);
if (eta=='')
    eta=2;
end
phi0=getValue(p,x);
epsilondphi0=d'*gx0*epsilon;


g=gx0'*d;
if(g==0)
    f0=getValue(p,x);
    res={x,f0};
    return;
end;

% initialization of the TPP
step=abs(atan(g)/g);
[alfa0,alfa1,alfa2,f0,f1,f2]=initTPP(p,x,d,step);

xk1=x+alfa1*d;
gx1=getGrad(p, xk1);    
gk1=gx1'*d;
if(gk1==0)
    fx1=getValue(p,x);
    res={x, fx1};
    return;
end;

if(alfa2==0)
	fxk2=getValue(p,x);
	res={x,fxk2};     
	return;
end
xk2=x+alfa2*d;
gx2=getGrad(p, xk2);    
if (maxiter==1 || gx2'*gx2==0)
    fxk2=getValue(p,xk2);
    res={xk2,fxk2};
    return;
end
n=2;
alfaEta=alfa1*eta;
phiEta=getValue(p,x+alfa1*eta*d);
while n<maxiter && ((f1 > phi0+epsilondphi0*alfa1) || (phiEta <= phi0+epsilondphi0*alfaEta ))
    gk1=gx1'*d;
    gk2=gx2'*d;
    deltaAlfa=alfa2-alfa1;
    if(deltaAlfa==0)
        xk2=x+alfa2*d;
        f2=getValue(p,xk2);
        res={xk2,f2};  
        return;
    end
    g1k2=(gk2-gk1)/deltaAlfa;
    alfa3=alfa2-gk2/g1k2;        
    xk3=x+alfa3*d;
    gx3=getGrad(p,xk3);
    f3=getValue(p,xk3);
    [alfa0,alfa1,alfa2,f0,f1,f2,gx0,gx1,gx2]=nextTPPGrad(alfa0,alfa1,alfa2,alfa3,f0,f1,f2,f3,gx0,gx1,gx2,gx3);
    alfaEta=alfa1*eta;
    phiEta=getValue(p,x+alfa1*eta*d);
    n=n+1;
end
xk=x+alfa1*d;
fxk=getValue(p,xk);
res={xk,fxk};
return;
