function res = wolfe(p,x,d,gx)
%function res = wolfe(p,x,d,gx)
%
% Funzione per la minimizzazione line search con il criterio di Wolfe
%
% res e' un cell array della forma {a,b} dove a e' il punto
% trovato, e b=f(a)

if (gx'*gx==0)
    fx=getValue(p,x);
    res={x,fx};
    return;
end

epsilon=getEpsilonArmijo(p);
if (epsilon=='')
    epsilon=.5;
end
eta=getEtaArmijo(p);
if (eta=='')
    eta=2;
end

alpha=0.1;
maxiter=getMaxiter(p);
tol=getTol(p);

phi0=getValue(p,x);
epsilondphi0=d'*gx*epsilon;
antiepsilondphi0=d'*gx*(1-epsilon);
xalpha=x+alpha*d;
phiAlpha=getValue(p,xalpha);
minAlpha=0;
minAlphaPhi=phi0;
maxAlpha=alpha;
maxAlphaPhi=phiAlpha;
while phiAlpha <= phi0+epsilondphi0*alpha;
    minAlpha=alpha;
    minAlphaPhi=phiAlpha;
    alpha=eta*alpha;    
    xalpha=x+alpha*d;
    phiAlpha=getValue(p,xalpha);
    maxAlpha=alpha;
    maxAlphaPhi=phiAlpha;
end

while phiAlpha > phi0+epsilondphi0*alpha;
    maxAlpha=alpha;
    maxAlphaPhi=phiAlpha;
    alpha=alpha/eta;    
    xalpha=x+alpha*d;
    phiAlpha=getValue(p,xalpha);
    minAlpha=alpha;
    minAlphaPhi=phiAlpha;
end

% a questo punto abbiamo che la condizione è soddisfatta senz'altro in un punto fra minalpha e
% maxalpha, partiamo con una ricerca con il metodo di bisezione
xalpha=x+maxAlpha*d;
maxAlphaGrad=getGrad(p,xalpha)'*d;
if ((maxAlphaPhi <= phi0+epsilondphi0*maxAlpha) && (maxAlphaGrad>=antiepsilondphi0))    
    res={xalpha,maxAlphaPhi};
    return;
end
xalpha=x+minAlpha*d;
minAlphaGrad=getGrad(p,xalpha)'*d;
if ((minAlphaPhi <= phi0+epsilondphi0*maxAlpha) && (minAlphaGrad>=antiepsilondphi0))
    xalpha=x+minAlpha*d;
    res={xalpha,minAlphaPhi};
    return;
end

n=0;
while ((n<maxiter)&&(maxAlpha-minAlpha>tol))
    alpha=(maxAlpha+minAlpha)/2.0;
    xalpha=x+alpha*d;
    phiAlpha=getValue(p,xalpha);
    phiGrad=getGrad(p,xalpha);
    % if goldstein rule holds
    if (phiGrad>antiepsilondphi0)
        if (phiAlpha <= phi0+epsilondphi0*alpha)
            % both rules hold: found        
            break;
        else
            maxAlpha=alpha;
        end
    else
        minAlpha=alpha;
    end
    n=n+1;
end

xalpha=x+alpha*d;
res={xalpha,phiAlpha};
return;
